@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,23 +13,21 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import warnings
-from typing import Callable, Dict, Optional, Sequence, Tuple
+from typing import Callable, Dict, Optional, Sequence, Tuple, Union
 
 from google.api_core import grpc_helpers  # type: ignore
 from google.api_core import operations_v1  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
-from google import auth  # type: ignore
-from google.auth import credentials  # type: ignore
+import google.auth  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 
 import grpc  # type: ignore
 
 from google.cloud.managedidentities_v1.types import managed_identities_service
 from google.cloud.managedidentities_v1.types import resource
-from google.longrunning import operations_pb2 as operations  # type: ignore
-
+from google.longrunning import operations_pb2  # type: ignore
 from .base import ManagedIdentitiesServiceTransport, DEFAULT_CLIENT_INFO
 
 
@@ -87,7 +84,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
         self,
         *,
         host: str = "managedidentities.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: str = None,
         scopes: Sequence[str] = None,
         channel: grpc.Channel = None,
@@ -101,7 +98,8 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -143,7 +141,10 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
           google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
               and ``credentials_file`` are passed.
         """
+        self._grpc_channel = None
         self._ssl_channel_credentials = ssl_channel_credentials
+        self._stubs: Dict[str, Callable] = {}
+        self._operations_client = None
 
         if api_mtls_endpoint:
             warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
@@ -151,95 +152,65 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
             warnings.warn("client_cert_source is deprecated", DeprecationWarning)
 
         if channel:
-            # Sanity check: Ensure that channel and credentials are not both
-            # provided.
+            # Ignore credentials if a channel was passed.
             credentials = False
-
             # If a channel was explicitly provided, set it.
             self._grpc_channel = channel
             self._ssl_channel_credentials = None
-        elif api_mtls_endpoint:
-            host = (
-                api_mtls_endpoint
-                if ":" in api_mtls_endpoint
-                else api_mtls_endpoint + ":443"
-            )
 
-            if credentials is None:
-                credentials, _ = auth.default(
-                    scopes=self.AUTH_SCOPES, quota_project_id=quota_project_id
-                )
-
-            # Create SSL credentials with client_cert_source or application
-            # default SSL credentials.
-            if client_cert_source:
-                cert, key = client_cert_source()
-                ssl_credentials = grpc.ssl_channel_credentials(
-                    certificate_chain=cert, private_key=key
-                )
-            else:
-                ssl_credentials = SslCredentials().ssl_credentials
-
-            # create a new channel. The provided one is ignored.
-            self._grpc_channel = type(self).create_channel(
-                host,
-                credentials=credentials,
-                credentials_file=credentials_file,
-                ssl_credentials=ssl_credentials,
-                scopes=scopes or self.AUTH_SCOPES,
-                quota_project_id=quota_project_id,
-                options=[
-                    ("grpc.max_send_message_length", -1),
-                    ("grpc.max_receive_message_length", -1),
-                ],
-            )
-            self._ssl_channel_credentials = ssl_credentials
         else:
-            host = host if ":" in host else host + ":443"
+            if api_mtls_endpoint:
+                host = api_mtls_endpoint
 
-            if credentials is None:
-                credentials, _ = auth.default(
-                    scopes=self.AUTH_SCOPES, quota_project_id=quota_project_id
-                )
+                # Create SSL credentials with client_cert_source or application
+                # default SSL credentials.
+                if client_cert_source:
+                    cert, key = client_cert_source()
+                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
+                        certificate_chain=cert, private_key=key
+                    )
+                else:
+                    self._ssl_channel_credentials = SslCredentials().ssl_credentials
 
-            if client_cert_source_for_mtls and not ssl_channel_credentials:
-                cert, key = client_cert_source_for_mtls()
-                self._ssl_channel_credentials = grpc.ssl_channel_credentials(
-                    certificate_chain=cert, private_key=key
-                )
+            else:
+                if client_cert_source_for_mtls and not ssl_channel_credentials:
+                    cert, key = client_cert_source_for_mtls()
+                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
+                        certificate_chain=cert, private_key=key
+                    )
 
-            # create a new channel. The provided one is ignored.
-            self._grpc_channel = type(self).create_channel(
-                host,
-                credentials=credentials,
-                credentials_file=credentials_file,
-                ssl_credentials=self._ssl_channel_credentials,
-                scopes=scopes or self.AUTH_SCOPES,
-                quota_project_id=quota_project_id,
-                options=[
-                    ("grpc.max_send_message_length", -1),
-                    ("grpc.max_receive_message_length", -1),
-                ],
-            )
-
-        self._stubs = {}  # type: Dict[str, Callable]
-        self._operations_client = None
-
-        # Run the base constructor.
+        # The base transport sets the host, credentials and scopes
         super().__init__(
             host=host,
             credentials=credentials,
             credentials_file=credentials_file,
-            scopes=scopes or self.AUTH_SCOPES,
+            scopes=scopes,
             quota_project_id=quota_project_id,
             client_info=client_info,
         )
 
+        if not self._grpc_channel:
+            self._grpc_channel = type(self).create_channel(
+                self._host,
+                credentials=self._credentials,
+                credentials_file=credentials_file,
+                scopes=self._scopes,
+                ssl_credentials=self._ssl_channel_credentials,
+                quota_project_id=quota_project_id,
+                options=[
+                    ("grpc.max_send_message_length", -1),
+                    ("grpc.max_receive_message_length", -1),
+                ],
+            )
+
+        # Wrap messages. This must be done after self._grpc_channel exists
+        self._prep_wrapped_messages(client_info)
+
     @classmethod
     def create_channel(
         cls,
         host: str = "managedidentities.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: str = None,
         scopes: Optional[Sequence[str]] = None,
         quota_project_id: Optional[str] = None,
@@ -247,7 +218,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
     ) -> grpc.Channel:
         """Create and return a gRPC channel object.
         Args:
-            address (Optional[str]): The host for the channel to use.
+            host (Optional[str]): The host for the channel to use.
             credentials (Optional[~.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify this application to the service. If
@@ -270,13 +241,15 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
             google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
               and ``credentials_file`` are passed.
         """
-        scopes = scopes or cls.AUTH_SCOPES
+
+        self_signed_jwt_kwargs = cls._get_self_signed_jwt_kwargs(host, scopes)
+
         return grpc_helpers.create_channel(
             host,
             credentials=credentials,
             credentials_file=credentials_file,
-            scopes=scopes,
             quota_project_id=quota_project_id,
+            **self_signed_jwt_kwargs,
             **kwargs,
         )
 
@@ -305,7 +278,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
         self,
     ) -> Callable[
         [managed_identities_service.CreateMicrosoftAdDomainRequest],
-        operations.Operation,
+        operations_pb2.Operation,
     ]:
         r"""Return a callable for the create microsoft ad domain method over gRPC.
 
@@ -325,7 +298,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
             self._stubs["create_microsoft_ad_domain"] = self.grpc_channel.unary_unary(
                 "/google.cloud.managedidentities.v1.ManagedIdentitiesService/CreateMicrosoftAdDomain",
                 request_serializer=managed_identities_service.CreateMicrosoftAdDomainRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["create_microsoft_ad_domain"]
 
@@ -417,7 +390,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
     def update_domain(
         self,
     ) -> Callable[
-        [managed_identities_service.UpdateDomainRequest], operations.Operation
+        [managed_identities_service.UpdateDomainRequest], operations_pb2.Operation
     ]:
         r"""Return a callable for the update domain method over gRPC.
 
@@ -437,7 +410,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
             self._stubs["update_domain"] = self.grpc_channel.unary_unary(
                 "/google.cloud.managedidentities.v1.ManagedIdentitiesService/UpdateDomain",
                 request_serializer=managed_identities_service.UpdateDomainRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["update_domain"]
 
@@ -445,7 +418,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
     def delete_domain(
         self,
     ) -> Callable[
-        [managed_identities_service.DeleteDomainRequest], operations.Operation
+        [managed_identities_service.DeleteDomainRequest], operations_pb2.Operation
     ]:
         r"""Return a callable for the delete domain method over gRPC.
 
@@ -465,7 +438,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
             self._stubs["delete_domain"] = self.grpc_channel.unary_unary(
                 "/google.cloud.managedidentities.v1.ManagedIdentitiesService/DeleteDomain",
                 request_serializer=managed_identities_service.DeleteDomainRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["delete_domain"]
 
@@ -473,7 +446,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
     def attach_trust(
         self,
     ) -> Callable[
-        [managed_identities_service.AttachTrustRequest], operations.Operation
+        [managed_identities_service.AttachTrustRequest], operations_pb2.Operation
     ]:
         r"""Return a callable for the attach trust method over gRPC.
 
@@ -493,7 +466,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
             self._stubs["attach_trust"] = self.grpc_channel.unary_unary(
                 "/google.cloud.managedidentities.v1.ManagedIdentitiesService/AttachTrust",
                 request_serializer=managed_identities_service.AttachTrustRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["attach_trust"]
 
@@ -501,7 +474,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
     def reconfigure_trust(
         self,
     ) -> Callable[
-        [managed_identities_service.ReconfigureTrustRequest], operations.Operation
+        [managed_identities_service.ReconfigureTrustRequest], operations_pb2.Operation
     ]:
         r"""Return a callable for the reconfigure trust method over gRPC.
 
@@ -521,7 +494,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
             self._stubs["reconfigure_trust"] = self.grpc_channel.unary_unary(
                 "/google.cloud.managedidentities.v1.ManagedIdentitiesService/ReconfigureTrust",
                 request_serializer=managed_identities_service.ReconfigureTrustRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["reconfigure_trust"]
 
@@ -529,7 +502,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
     def detach_trust(
         self,
     ) -> Callable[
-        [managed_identities_service.DetachTrustRequest], operations.Operation
+        [managed_identities_service.DetachTrustRequest], operations_pb2.Operation
     ]:
         r"""Return a callable for the detach trust method over gRPC.
 
@@ -549,7 +522,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
             self._stubs["detach_trust"] = self.grpc_channel.unary_unary(
                 "/google.cloud.managedidentities.v1.ManagedIdentitiesService/DetachTrust",
                 request_serializer=managed_identities_service.DetachTrustRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["detach_trust"]
 
@@ -557,7 +530,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
     def validate_trust(
         self,
     ) -> Callable[
-        [managed_identities_service.ValidateTrustRequest], operations.Operation
+        [managed_identities_service.ValidateTrustRequest], operations_pb2.Operation
     ]:
         r"""Return a callable for the validate trust method over gRPC.
 
@@ -579,7 +552,7 @@ class ManagedIdentitiesServiceGrpcTransport(ManagedIdentitiesServiceTransport):
             self._stubs["validate_trust"] = self.grpc_channel.unary_unary(
                 "/google.cloud.managedidentities.v1.ManagedIdentitiesService/ValidateTrust",
                 request_serializer=managed_identities_service.ValidateTrustRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["validate_trust"]
 
